@@ -31,6 +31,7 @@ DBHandle *db_handle_new(gchar *root) {
     g_free(self);
     return NULL;
   }
+  self->upcount = 1;
   return self;
 }
 
@@ -46,13 +47,19 @@ void db_handle_free(DBHandle *self) {
  */
 /* closes the db handle, if it hasn't been already */
 void db_handle_db_down(DBHandle *self) {
-  if (self->db) rpmdbClose(self->db);
-  self->db = NULL;
+  self->upcount--;
+  if (self->upcount == 0) {
+    if (self->db) rpmdbClose(self->db);
+    self->db = NULL;
+  }
 }
 
 /* opens it again (if it isn't already open) */
 void db_handle_db_up(DBHandle *self) {
-  if (self->db) return;
-  if (rpmdbOpen(self->root, &self->db, O_RDONLY, 0644))
-    self->db = NULL;
+  self->upcount++;
+  if (self->upcount == 1)
+    if (rpmdbOpen(self->root, &self->db, O_RDONLY, 0644)) {
+      g_warning("Could not open database");
+      self->db = NULL;
+    }
 }
