@@ -21,10 +21,13 @@
 #endif
 
 #include "search.h"
+#include <string.h>
 #include <stdlib.h>
 #include <unistd.h>
 #include <libgnome/libgnome.h>
 #include <time.h>
+/* this is needed for use of the SAX XML parser interface */
+#include <parserInternals.h>
 #include "rdf_api.h"
 #include "rdf.h"
 #include "trans.h"
@@ -38,200 +41,296 @@
 /* undef this if you don't want to cache rpmData structures */
 #define RPM_DATA_CACHE
 
-static GList *allPackages = NULL;
 extern GVoidFunc libfind_idle_func;
 
-GList *getFullPackageList(void) {
-  rdfSchema rdf;
-  rdfNamespace rdfNs, rpmNs;
-  rdfDescription desc;
-  char *file = update_file("resources/fullIndex.rdf.gz");
-  GList *tail = NULL;
-  GHashTable *check;
+/* --- These functions form a SAX parser for the fullIndex.rdf index -- */
+/* (we use this instead of the other RDF interface because the memory
+ *  usage skyrockets when we use it to load 3.5Mbytes of XML) */
+typedef enum {
+  PARSER_START,
+  PARSER_IN_RDF,
+  PARSER_IN_DESCRIPTION,
+  PARSER_IN_NAME,
+  PARSER_IN_SUMMARY,
+} ParserState;
 
-  if (allPackages != NULL)
-    return allPackages;
+typedef struct _SearchParseState {
+  /* tag names -- when the xmlnamespaces, the correct prefixes are
+   * prepended to the tags, and saved here for later comparison */
+  gchar *rdf, *description, *name, *summary;
+  GString *nameData, *summaryData;
+  GList *results, *tail;
 
-  rdf = rdfRead(file);
-  if (rdf == NULL) {
-    g_warning("can't open catalog '%s'", file);
-    g_free(file);
-    return NULL;
-  }
-  rdfNs = rdfGetNamespace(rdf, "http://www.w3.org/TR/WD-rdf-syntax#");
-  if (rdfNs == NULL) {
-    g_warning("%s is not an RDF schema", file);
-    rdfDestroySchema(rdf);
-    g_free(file);
-    return NULL;
-  }
-  rpmNs = rdfGetNamespace(rdf, "http://www.rpm.org/");
-  if (rpmNs == NULL) {
-    g_warning("%s is not an RPM specific RDF schema", file);
-    rdfDestroySchema(rdf);
-    g_free(file);
-    return NULL;
-  }
-  desc = rdfFirstDescription(rdf);
-  if (desc == NULL) {
-    g_warning("%s RDF schema seems empty", file);
-    rdfDestroySchema(rdf);
-    g_free(file);
-    return NULL;
-  }
-
-  check = g_hash_table_new(g_str_hash, g_str_equal);
-  while (desc != NULL) {
-    char *name;
-
-    rdfGetValue(desc, "Name", rpmNs, &name, NULL);
-    if (!g_hash_table_lookup(check, name)) {
-      char *key = g_strdup(name);
-
-      if (tail) {
-	g_list_append(tail, key);
-	tail = g_list_last(tail);
-      } else
-	allPackages = tail = g_list_append(NULL, key);
-      g_hash_table_insert(check, key, GINT_TO_POINTER(TRUE));
-    }
-    if (name) free(name);
-    desc = rdfNextDescription(desc);
-  }
-  g_list_sort(allPackages, (GCompareFunc)g_strcasecmp);
-  g_hash_table_destroy(check);
-  return allPackages;
-}
-
-GList *aproposSearch(const char *search) {
-  GList *ret = NULL, *tail1 = NULL, *tail2 = NULL;
-  rdfSchema rdf;
-  rdfNamespace rdfNs, rpmNs;
-  rdfDescription desc;
-  gboolean buildPackages = FALSE;
-  GHashTable *check = NULL;
+  /* this is for the search interface ... */
+  const gchar *search; /* this is NULL if we want all items */
 #ifdef HAVE_REGEX_H
   regex_t reg;
-  regmatch_t pmatch;
 #endif
-  char *file = update_file("resources/fullIndex.rdf.gz");
 
-  if (!allPackages)
-    buildPackages = TRUE;
+  ParserState state;
+  GHashTable *dupCheck;
+} SearchParseState;
 
-  g_message("Searching for '%s'", search);
-  rdf = rdfRead(file);
-  if (rdf == NULL) {
-    g_warning("can't open catalog '%s'", file);
-    g_free(file);
-    return NULL;
-  }
-  rdfNs = rdfGetNamespace(rdf, "http://www.w3.org/TR/WD-rdf-syntax#");
-  if (rdfNs == NULL) {
-    g_warning("%s is not an RDF schema", file);
-    rdfDestroySchema(rdf);
-    g_free(file);
-    return NULL;
-  }
-  rpmNs = rdfGetNamespace(rdf, "http://www.rpm.org/");
-  if (rpmNs == NULL) {
-    g_warning("%s is not an RPM specific RDF schema", file);
-    rdfDestroySchema(rdf);
-    g_free(file);
-    return NULL;
-  }
-  desc = rdfFirstDescription(rdf);
-  if (desc == NULL) {
-    g_warning("%s RDF schema seems empty", file);
-    rdfDestroySchema(rdf);
-    g_free(file);
-    return NULL;
-  }
+static void searchStartDocument(SearchParseState *state) {
+  state->rdf = state->description = state->name = state->summary = NULL;
+
+  state->state = PARSER_START;
+  state->nameData = g_string_sized_new(128);
+  state->summaryData = g_string_sized_new(256);
+
+  state->results = state->tail = NULL;
+
+  /* we only bother with a hash table for dup checking when listing all
+   * packages */
+  if (!state->search)
+    state->dupCheck = g_hash_table_new(g_str_hash, g_str_equal);
 
 #ifdef HAVE_REGEX_H
+  if (state->search) {
 #ifdef REG_ICASE
 # ifdef REG_NOSUB
-  regcomp(&reg, search, REG_ICASE|REG_NOSUB);
+    regcomp(&(state->reg), state->search, REG_ICASE|REG_NOSUB);
 # else
-  regcomp(&reg, search, REG_ICASE);
+    regcomp(&(state->reg), state->search, REG_ICASE);
 # endif
 #else
 # ifdef REG_NOSUB
-  regcomp(&reg, search, REG_NOSUB);
+    regcomp(&(state->reg), state->search, REG_NOSUB);
 # else
-  regcomp(&reg, search, 0);
+    regcomp(&(state->reg), state->search, 0);
 # endif
 #endif
+  }
 #endif
-
-  if (buildPackages)
-    check = g_hash_table_new(g_str_hash, g_str_equal);
-
-  while (desc != NULL) {
-    char *URL, *name, *description;
-
-    URL = rdfGetDescriptionAbout(rdf, desc);
-    if (URL == NULL) {
-      g_warning("%s RDF schema invalid: description without href", file);
-      rdfDestroySchema(rdf);
-      g_free(file);
-      return NULL;
-    }
-    rdfGetValue(desc, "Name", rpmNs, &name, NULL);
-    rdfGetValue(desc, "Summary", rpmNs, &description, NULL);
-
-    if (buildPackages)
-      if (!g_hash_table_lookup(check, name)) {
-	char *key = g_strdup(name);
-
-	if (tail1) {
-	  g_list_append(tail1, key);
-	  tail1 = g_list_last(tail1);
-	} else
-	  allPackages = tail1 = g_list_append(NULL, key);
-	g_hash_table_insert(check, key, GINT_TO_POINTER(TRUE));
-      }
+}
+static void searchEndDocument(SearchParseState *state) {
+  g_free(state->rdf);
+  g_free(state->description);
+  g_free(state->name);
+  g_free(state->summary);
+  g_string_free(state->nameData, TRUE);
+  g_string_free(state->summaryData, TRUE);
+  if (!state->search)
+    g_hash_table_destroy(state->dupCheck);
+  state->tail = NULL;
 
 #ifdef HAVE_REGEX_H
-    if ((name && !regexec(&reg, name, 1, &pmatch, 0)) ||
-	(description && !regexec(&reg, description, 1, &pmatch, 0)))
+  if (state->search)
+    regfree(&(state->reg));
+#endif
+}
+static void searchStartElement(SearchParseState *state, const CHAR *name,
+			       const CHAR **attrs) {
+  int i;
+
+  switch (state->state) {
+  case PARSER_START:
+    for (i = 0; attrs[i] != NULL; i++) {
+      const CHAR *aname = attrs[i++], *adata = attrs[i];
+      if (!strncmp(aname, "xmlns:", 6)) {
+	aname += 6;
+	if (!strcmp(adata, "http://www.w3.org/TR/WD-rdf-syntax#")) {
+	  g_free(state->rdf);
+	  state->rdf = g_strconcat(aname, ":RDF", NULL);
+	  g_free(state->description);
+	  state->description = g_strconcat(aname, ":Description", NULL);
+	} else if (!strcmp(adata, "http://www.rpm.org/")) {
+	  g_free(state->name);
+	  state->name = g_strconcat(aname, ":Name", NULL);
+	  g_free(state->summary);
+	  state->summary = g_strconcat(aname, ":Summary", NULL);
+	}
+      }
+    }
+    g_assert(state->rdf != NULL);
+    g_assert(state->description != NULL);
+    g_assert(state->name != NULL);
+    g_assert(state->summary != NULL);
+    if (strcmp(name, state->rdf))
+      g_warning("expected %s as toplevel element -- got %s",state->rdf,name);
+
+    state->state = PARSER_IN_RDF;
+    break;
+  case PARSER_IN_RDF:
+    if (strcmp(name, state->description))
+      g_warning("was not expecting a %s element", name);
+    g_string_truncate(state->nameData, 0);
+    g_string_truncate(state->summaryData, 0);
+    state->state = PARSER_IN_DESCRIPTION;
+    break;
+  case PARSER_IN_DESCRIPTION:
+    if (!strcmp(name, state->name)) {
+      g_string_truncate(state->nameData, 0);
+      state->state = PARSER_IN_NAME;
+    } else if (!strcmp(name, state->summary)) {
+      g_string_truncate(state->summaryData, 0);
+      state->state = PARSER_IN_SUMMARY;
+    } else
+      g_warning("was not expecting a %s element", name);
+    break;
+  case PARSER_IN_NAME:
+  case PARSER_IN_SUMMARY:
+    g_warning("was not expecting a %s element", name);
+    break;
+  }
+}
+static void searchEndElement(SearchParseState *state, const CHAR *name) {
+  switch (state->state) {
+  case PARSER_START:
+    g_warning("This state should not have been reached");
+    break;
+  case PARSER_IN_RDF:
+    break;
+  case PARSER_IN_DESCRIPTION:
+    //    g_message("End description for %s", state->nameData->str);
+    if (state->search) {
+      regmatch_t pmatch;
+      gchar *name = state->nameData->str;
+      gchar *summary = state->summaryData->str;
+
+      if (
+#ifdef HAVE_REGEX_H
+	  (name && !regexec(&(state->reg), name, 1, &pmatch, 0)) ||
+	  (summary && !regexec(&(state->reg), summary, 1, &pmatch, 0)) ||
 #elif defined(HAVE_STRSTR)
-    if ((name && strstr(name, search)) ||
-	(description && strstr(description, search)))
+	  (name && strstr(name, state->search)) ||
+	  (summary && strstr(summar, state->search)) ||
 #else
 #error "You need regex code or strstr for this to work"
 #endif
-      if (!g_list_find_custom(ret, name, (GCompareFunc)g_strcasecmp)) {
-	if (tail2) {
-	  g_list_append(tail2, g_strdup(name));
-	  tail2 = g_list_last(tail2);
+	  FALSE)
+	if (!g_list_find_custom(state->results, name, (GCompareFunc)strcmp)) {
+	  if (state->tail) {
+	    g_list_append(state->tail, g_strdup(name));
+	    state->tail = g_list_last(state->tail);
+	  } else
+	    state->results = state->tail = g_list_append(NULL, g_strdup(name));
+	}
+    } else {
+      if (!g_hash_table_lookup(state->dupCheck, state->nameData->str)) {
+	gchar *key = g_strdup(state->nameData->str);
+
+	if (state->tail) {
+	  g_list_append(state->tail, key);
+	  state->tail = g_list_last(state->tail);
 	} else
-	  ret = tail2 = g_list_append(NULL, g_strdup(name));
+	  state->results = state->tail = g_list_append(NULL, key);
+	g_hash_table_insert(state->dupCheck, key, GINT_TO_POINTER(TRUE));
       }
-    free(URL);
-    if (name) free(name);
-    if (description) free(description);
-
-    if (libfind_idle_func)
-      (* libfind_idle_func)();
-
-    desc = rdfNextDescription(desc);
+    }
+    state->state = PARSER_IN_RDF;
+    break;
+  case PARSER_IN_NAME:
+  case PARSER_IN_SUMMARY:
+    state->state = PARSER_IN_DESCRIPTION;
+    break;
   }
-  rdfDestroySchema(rdf);
+}
+static void searchCharacters(SearchParseState *state, const CHAR *chars,int len) {
+  int i;
 
-  if (buildPackages) {
-    g_hash_table_destroy(check);
-    g_list_sort(allPackages, (GCompareFunc)g_strcasecmp);
+  switch (state->state) {
+  case PARSER_IN_NAME:
+    for (i = 0; i < len; i++)
+      g_string_append_c(state->nameData, chars[i]);
+    break;
+  case PARSER_IN_SUMMARY:
+    for (i = 0; i < len; i++)
+      g_string_append_c(state->summaryData, chars[i]);
+    break;
+  default:
+    break;
   }
+}
 
-  g_free(file);
-#ifdef HAVE_REGEX_H
-  regfree(&reg);
-#endif
+static xmlEntityPtr searchGetEntity(SearchParseState *state, const CHAR *name) {
+  return xmlGetPredefinedEntity(name);
+}
 
-  ret = g_list_sort(ret, (GCompareFunc)g_strcasecmp);
+static void searchComment(SearchParseState *state, const char *msg) {
+  g_log("XML", G_LOG_LEVEL_MESSAGE, "%s", msg);
+}
 
-  return ret;
+static void searchWarning(SearchParseState *state, const char *msg, ...) {
+  va_list args;
+
+  va_start(args, msg);
+  g_logv("XML", G_LOG_LEVEL_WARNING, msg, args);
+  va_end(args);
+}
+
+static void searchError(SearchParseState *state, const char *msg, ...) {
+  va_list args;
+
+  va_start(args, msg);
+  g_logv("XML", G_LOG_LEVEL_CRITICAL, msg, args);
+  va_end(args);
+}
+
+static void searchFatalError(SearchParseState *state, const char *msg, ...) {
+  va_list args;
+
+  va_start(args, msg);
+  g_logv("XML", G_LOG_LEVEL_ERROR, msg, args);
+  va_end(args);
+}
+
+static xmlSAXHandler searchSAXParser = {
+   0, /* internalSubset */
+   0, /* isStandalone */
+   0, /* hasInternalSubset */
+   0, /* hasExternalSubset */
+   0, /* resolveEntity */
+   (getEntitySAXFunc)searchGetEntity, /* getEntity */
+   0, /* entityDecl */
+   0, /* notationDecl */
+   0, /* attributeDecl */
+   0, /* elementDecl */
+   0, /* unparsedEntityDecl */
+   0, /* setDocumentLocator */
+   (startDocumentSAXFunc)searchStartDocument, /* startDocument */
+   (endDocumentSAXFunc)searchEndDocument, /* endDocument */
+   (startElementSAXFunc)searchStartElement, /* startElement */
+   (endElementSAXFunc)searchEndElement, /* endElement */
+   0, /* reference */
+   (charactersSAXFunc)searchCharacters, /* characters */
+   0, /* ignorableWhitespace */
+   0, /* processingInstruction */
+   (commentSAXFunc)searchComment, /* comment */
+   (warningSAXFunc)searchWarning, /* warning */
+   (errorSAXFunc)searchError, /* error */
+   (fatalErrorSAXFunc)searchFatalError, /* fatalError */
+};
+
+GList *aproposSearch(const gchar *search) {
+  xmlParserCtxtPtr ctxt;
+  SearchParseState state;
+  char *filename = update_file("resources/fullIndex.rdf.gz");
+
+  state.search = search;
+
+  ctxt = xmlCreateFileParserCtxt(filename);
+  g_free(filename);
+  if (ctxt == NULL) return NULL;
+  ctxt->sax = &searchSAXParser;
+  ctxt->userData = &state;
+
+  xmlParseDocument(ctxt);
+
+  if (!ctxt->wellFormed) {
+    g_list_foreach(state.results, (GFunc)g_free, NULL);
+    g_list_free(state.results);
+    state.results = NULL;
+  }
+  ctxt->sax = NULL;
+  xmlFreeParserCtxt(ctxt);
+
+  return g_list_sort(state.results, (GCompareFunc)g_strcasecmp);
+}
+
+
+GList *getFullPackageList(void) {
+  return aproposSearch(NULL);
 }
 
 void freeApropos(GList *apropos) {
@@ -321,6 +420,7 @@ GList *alternateSearch(const char *resource) {
     }
     
     alt = g_new0(rpmPackageAlternate, 1);
+    alt->status = UNKNOWN_PACKAGE;
     rdfGetValue(desc, "Name", rpmNs, &value, NULL);
     if (value != NULL) {
       alt->name = g_strdup(value);
@@ -495,6 +595,65 @@ void freeAlternates(GList *alternates) {
   g_list_free(alternates);
 }
 
+void statAlternates(rpmdb db, GList *alternates) {
+  Header installed = NULL;
+  GList *tmp;
+  gchar *prevname = NULL;
+
+  for (tmp = alternates; tmp; tmp = tmp->next) {
+    rpmPackageAlternate *alt = tmp->data;
+
+    /* a slight optimisation, since usually all alternates have the same name*/
+    if (!prevname || strcmp(prevname, alt->name) != 0) {
+      dbiIndexSet matches;
+    
+      if (installed) headerFree(installed);
+      installed = NULL;
+      if (!rpmdbFindByLabel(db, alt->name, &matches)) {
+	gint i;
+
+	for (i = 0; i < matches.count; i++) {
+	  gint index = matches.recs[i].recOffset;
+	  Header tmp = rpmdbGetRecord(db, index);
+
+	  if (!installed) installed = tmp;
+	  else {
+	    if (rpmVersionCompare(installed, tmp) < 0) {
+	      headerFree(installed);
+	      installed = tmp;
+	    } else
+	      headerFree(tmp);
+	  }
+	}
+	dbiFreeIndexRecord(matches);
+      }
+    }
+    /* installed holds the header for the newest version of the package,
+     * or is NULL if the package is not installed */
+    if (!installed)
+      alt->status = UNKNOWN_PACKAGE;
+    else {
+      gint res;
+      Header this = headerNew();
+
+      headerAddEntry(this, RPMTAG_NAME,    RPM_STRING_TYPE, alt->name, 1);
+      headerAddEntry(this, RPMTAG_VERSION, RPM_STRING_TYPE, alt->version, 1);
+      headerAddEntry(this, RPMTAG_RELEASE, RPM_STRING_TYPE, alt->release, 1);
+
+      res = rpmVersionCompare(this, installed);
+      if (res < 0)
+	alt->status = OLD_PACKAGE;
+      else if (res == 0)
+	alt->status = CURRENT_PACKAGE;
+      else
+	alt->status = NEW_PACKAGE;
+      headerFree(this);
+    }
+    prevname = alt->name;
+  }
+  if (installed) headerFree(installed);
+}
+
 void listAlternates(GList *alternates) {
   GList *tmp;
 
