@@ -174,28 +174,29 @@ static gint installPackages(gchar *rootdir, gchar **packages, gchar *location,
 }
 #endif
 
-/* returns TRUE if yes was pressed */
-static gboolean printDepProblems(struct rpmDependencyConflict *conflicts,
-                            int numConflicts) {
-  GtkWidget *win, *w, *list;
-  gchar buf[1024];
-  gint i, flags, ret = FALSE;
-  GList *items = NULL;
+static void addDepNodes(rpmdb db, gchar *rootdir,
+			GtkCTree *ctree, GtkCTreeNode *parent,
+			struct rpmDependencyConflict *conflicts,
+			int numConflicts, GList **processed) {
+  GtkCTreeNode *node;
+  gint i, flags;
+  guint32 offs;
+  gchar buf[1024], *row[1];
+  dbiIndexSet matches;
+  struct rpmDependencyConflict *conf2;
+  gint numConf2;
 
-  win = gnome_dialog_new(_("Dependency Problems"), GNOME_STOCK_BUTTON_YES,
-			 GNOME_STOCK_BUTTON_NO, NULL);
-  gnome_dialog_close_hides(GNOME_DIALOG(win), FALSE);
-  gtk_window_set_policy(GTK_WINDOW(win), FALSE, TRUE, TRUE);
-  set_icon(win);
-  gtk_widget_set_usize(win, 300, -1);
-  gnome_dialog_set_default(GNOME_DIALOG(win), 1);
-  gnome_dialog_set_close(GNOME_DIALOG(win), TRUE);
-
-  w = gtk_label_new(_("The following dependency problems occured:"));
-  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(win)->vbox), w, FALSE, TRUE, 0);
-  gtk_widget_show(w);
+  row[0] = buf;
   for (i = 0; i < numConflicts; i++) {
+    /* construct the text for this conflict */
     buf[0] = '\0';
+    strcat(buf, conflicts[i].byName); strcat(buf, "-");
+    strcat(buf, conflicts[i].byVersion); strcat(buf, "-");
+    strcat(buf, conflicts[i].byRelease);
+    if (conflicts[i].sense == RPMDEP_SENSE_REQUIRES)
+      strcat(buf, _(" requires "));
+    else
+      strcat(buf, _(" conflicts with "));
     strcat(buf, conflicts[i].needsName);
     flags = conflicts[i].needsFlags;
     if (flags) {
@@ -211,28 +212,93 @@ static gboolean printDepProblems(struct rpmDependencyConflict *conflicts,
       strcat(buf, " ");
       strcat(buf, conflicts[i].needsVersion);
     }
-    if (conflicts[i].sense == RPMDEP_SENSE_REQUIRES)
-      strcat(buf, _(" is needed by "));
-    else
-      strcat(buf, _(" conflicts with "));
-    strcat(buf, conflicts[i].byName);    strcat(buf, "-");
-    strcat(buf, conflicts[i].byVersion); strcat(buf, "-");
-    strcat(buf, conflicts[i].byRelease);
-    w = gtk_list_item_new_with_label(buf);
-    gtk_widget_show(w);
-    items = g_list_append(items, w);
+    node = gtk_ctree_insert_node(ctree, parent, NULL, row, 0,
+				 NULL, NULL, NULL, NULL, FALSE, FALSE);
+
+    /* get the record offset for this package ... */
+    if (rpmdbFindByHeader(db, conflicts[i].byHeader, &matches))
+      /* could not find header ... */
+      continue;
+    offs = matches.recs[0].recOffset;
+    dbiFreeIndexRecord(matches);
+    if (!offs || g_list_find(*processed, GUINT_TO_POINTER(offs)))
+	/* already processed this package */
+	continue;
+    *processed = g_list_prepend(*processed, GUINT_TO_POINTER(offs));
+
+    /* get a conflicts array for this package */
+#ifdef HAVE_RPM_3_0
+    {
+      rpmTransactionSet rpmdep = rpmtransCreateSet(db, rootdir);
+      rpmtransRemovePackage(rpmdep, offs);
+      if (rpmdepCheck(rpmdep, &conf2, &numConf2)) {
+	rpmtransFree(rpmdep);
+	continue;
+      }
+      rpmtransFree(rpmdep);
+    }
+#else
+    {
+      rpmDependencies rpmdep = rpmdepDependencies(db);
+      rpmdepRemovePackage(rpmdep, offs);
+      if (rpmdepCheck(rpmdep, &conf2, &numConf2)) {
+	rpmdepDone(rpmdep);
+	continue;
+      }
+      rpmdepDone(rpmdep);
+    }
+#endif
+    /* add nodes for packages this one depends on ... */
+    addDepNodes(db, rootdir, ctree, node, conf2, numConf2, processed);
+    /* free the conflicts array */
+    rpmdepFreeConflicts(conf2, numConf2);
   }
+}
+
+/* returns TRUE if yes was pressed */
+static gboolean displayDepProblems(struct rpmDependencyConflict *conflicts,
+				   int numConflicts,
+				   rpmdb db, gchar *rootdir,
+				   GList **dependentPkgs) {
+  GtkWidget *win, *w, *ctree;
+  gchar buf[1024];
+  gint i, flags, ret = FALSE;
+  GList *processed = NULL;
+
+  win = gnome_dialog_new(_("Dependency Problems"), GNOME_STOCK_BUTTON_YES,
+			 GNOME_STOCK_BUTTON_NO, NULL);
+  gnome_dialog_close_hides(GNOME_DIALOG(win), FALSE);
+  gtk_window_set_policy(GTK_WINDOW(win), FALSE, TRUE, TRUE);
+  set_icon(win);
+  gtk_widget_set_usize(win, 300, -1);
+  gnome_dialog_set_default(GNOME_DIALOG(win), 1);
+  gnome_dialog_set_close(GNOME_DIALOG(win), TRUE);
+
+  w = gtk_label_new(_("The following dependency problems occured:"));
+  gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(win)->vbox), w, FALSE, TRUE, 0);
+  gtk_widget_show(w);
+
   w = gtk_scrolled_window_new(NULL, NULL);
-  gtk_widget_set_usize(w, 320, 120);
+  gtk_widget_set_usize(w, 400, 160);
   gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(w), GTK_POLICY_AUTOMATIC,
 				 GTK_POLICY_AUTOMATIC);
   gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(win)->vbox), w, TRUE, TRUE, 0);
   gtk_widget_show(w);
-  list = gtk_list_new();
-  gtk_list_set_selection_mode(GTK_LIST(list), GTK_SELECTION_BROWSE);
-  gtk_list_append_items(GTK_LIST(list), items);
-  gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(w), list);
-  gtk_widget_show(list);
+
+  ctree = gtk_ctree_new(1, 0);
+  gtk_ctree_set_indent(GTK_CTREE(ctree), 10);
+  gtk_clist_set_selection_mode(GTK_CLIST(ctree), GTK_SELECTION_BROWSE);
+  gtk_clist_set_column_auto_resize(GTK_CLIST(ctree), 0, TRUE);
+
+  addDepNodes(db, rootdir, GTK_CTREE(ctree), NULL,
+	      conflicts, numConflicts, &processed);
+  if (dependentPkgs == NULL)
+    g_list_free(processed);
+  else
+    *dependentPkgs = processed;
+
+  gtk_container_add(GTK_CONTAINER(w), ctree);
+  gtk_widget_show(ctree);
 
   w = gtk_label_new(_("Do you want to ignore these problems?\n"
 		      "(saying yes may make your system unstable)"));
@@ -374,7 +440,7 @@ int do_install(gchar *rootdir, GList *pkgs, gchar *location,
         stopInstall = 1;
       }
       if (!stopInstall && conflicts) {
-        if (!printDepProblems(conflicts, numConflicts)) {
+        if (!displayDepProblems(conflicts, numConflicts, db, rootdir, NULL)) {
 	  numFailed = numPackages;
 	  stopInstall = 1;
 	}
@@ -512,7 +578,7 @@ int do_install(gchar *rootdir, GList *pkgs, gchar *location,
         stopInstall = 1;
       }
       if (!stopInstall && conflicts) {
-        if (!printDepProblems(conflicts, numConflicts)) {
+        if (!displayDepProblems(conflicts, numConflicts, db, rootdir, NULL)) {
 	  numFailed = numPackages;
 	  stopInstall = 1;
 	}
@@ -620,7 +686,7 @@ gint do_uninstall(gchar *root, GList *indices, gint transFlags,
     }
 
     if (!stopUninstall && conflicts) {
-      if (!printDepProblems(conflicts, numConflicts)) {
+      if (!displayDepProblems(conflicts, numConflicts, db, root, NULL)) {
 	numFailed += g_list_length(indices);
 	stopUninstall = 1;
       }
@@ -665,7 +731,7 @@ gint do_uninstall(gchar *root, GList *indices, gint uninstallFlags,
     }
     rpmdepDone(rpmdep);
     if (!stopUninstall && conflicts) {
-      if (!printDepProblems(conflicts, numConflicts)) {
+      if (!displayDepProblems(conflicts, numConflicts, db, root, NULL)) {
 	numFailed += g_list_length(indices);
 	stopUninstall = 1;
       }
