@@ -240,9 +240,6 @@ static void create_menus(GtkWidget *app, GtkWidget *pl) {
     GNOMEUIINFO_ITEM_STOCK_DATA(N_("_Install..."),
 				N_("Install some new packages"),
 				rpm_install_pkgs, pl, RPM_STOCK_MENU_INSTALL),
-    GNOMEUIINFO_ITEM_STOCK_DATA(N_("_Upgrade..."),
-				N_("Upgrade some current packages"),
-				rpm_upgrade_pkgs, pl, RPM_STOCK_MENU_UPGRADE),
     GNOMEUIINFO_SEPARATOR,
     GNOMEUIINFO_ITEM_STOCK_DATA(N_("_Preferences..."),
 				N_("Alter GnoRPM's preferences"),
@@ -266,11 +263,8 @@ static void create_menus(GtkWidget *app, GtkWidget *pl) {
     GNOMEUIINFO_ITEM_STOCK_DATA(N_("Install"),
 				N_("Select packages for installation"),
                           rpm_install_pkgs, pl, RPM_STOCK_PIXMAP_INSTALL),
-    GNOMEUIINFO_ITEM_STOCK_DATA(N_("Upgrade"),
-				N_("Select packages for upgrade"),
-                          rpm_upgrade_pkgs, pl, RPM_STOCK_PIXMAP_UPGRADE),
     GNOMEUIINFO_SEPARATOR,
-    GNOMEUIINFO_ITEM_STOCK_DATA(N_("Unselect"), N_("Unselect all packags"),
+    GNOMEUIINFO_ITEM_STOCK_DATA(N_("Unselect"), N_("Unselect all packages"),
                           rpm_clear_selection, pl, RPM_STOCK_PIXMAP_UNSELECT),
     GNOMEUIINFO_SEPARATOR,
     GNOMEUIINFO_ITEM_STOCK_DATA(N_("Uninstall"),
@@ -303,14 +297,12 @@ static void rpm_clear_selection(GtkWidget *mi, RpmPackageList *pl) {
 }
 
 static void rpm_uninstall_pkgs(GtkWidget *mi, RpmPackageList *pl) {
-  db_handle_db_down(pl->hdl);
 #ifdef HAVE_RPM_3_0
   do_uninstall(pl->hdl->root, pl->selection, transFlags, probFilter,
 	       interfaceFlags);
 #else
   do_uninstall(pl->hdl->root, pl->selection, uninstallFlags, interfaceFlags);
 #endif
-  db_handle_db_up(pl->hdl);
   /* update the package list display */
   rpm_package_list_clear_selection(pl);
   rpm_package_list_update_pane(pl);
@@ -335,7 +327,6 @@ static void uninstall_func(RpmQuery *info, gpointer data) {
   char buf[512];
   RpmPackageList *pl = RPM_PACKAGE_LIST(data);
 
-  db_handle_db_down(info->hdl);
 #ifdef HAVE_RPM_3_0
   failed = uninstall_one(info->hdl->root, info->index, transFlags,
 			 probFilter, interfaceFlags);
@@ -343,7 +334,6 @@ static void uninstall_func(RpmQuery *info, gpointer data) {
   failed = uninstall_one(info->hdl->root, info->index, uninstallFlags,
 			 interfaceFlags);
 #endif
-  db_handle_db_up(info->hdl);
   if (failed) {
     g_snprintf(buf, 511, _("%d packages couldn't be uninstalled"), failed);
     message_box(buf);
@@ -410,14 +400,12 @@ static void rpm_find_uninstall_cb(RpmFindDialog *dlg, GList *indices,
   GList *tmp;
 
   if (indices == NULL) return;
-  db_handle_db_down(pl->hdl);
 #ifdef HAVE_RPM_3_0
   failed = do_uninstall(pl->hdl->root, indices, transFlags, probFilter,
 			interfaceFlags);
 #else
   failed = do_uninstall(pl->hdl->root, indices, uninstallFlags,interfaceFlags);
 #endif
-  db_handle_db_up(pl->hdl);
   if (failed) {
     g_snprintf(buf, 511, _("%d packages couldn't be uninstalled"), failed);
     message_box(buf);
@@ -454,7 +442,6 @@ static void gnorpm_update_tree(gchar *pkgName, gchar *groupName, void *data) {
 }
 
 static void install_func(RpmQuery *info, RpmPackageList *pl) {
-  db_handle_db_down(info->hdl);
 #ifdef HAVE_RPM_3_0
   if (install_one(info->hdl->root, info->fname, NULL, transFlags, probFilter,
 		  interfaceFlags, gnorpm_update_tree, pl)) {
@@ -466,11 +453,9 @@ static void install_func(RpmQuery *info, RpmPackageList *pl) {
     g_snprintf(buf, 511, _("Couldn't install %s"), info->fname);
     message_box(buf);
   }
-  db_handle_db_up(info->hdl);
   rpm_package_list_update_pane(pl);
 }
 static void upgrade_func(RpmQuery *info, RpmPackageList *pl) {
-  db_handle_db_down(info->hdl);
 #ifdef HAVE_RPM_3_0
   if (install_one(info->hdl->root, info->fname, NULL,
 		  transFlags, probFilter, interfaceFlags | INTER_UPGRADE,
@@ -484,11 +469,10 @@ static void upgrade_func(RpmQuery *info, RpmPackageList *pl) {
     g_snprintf(buf, 511, _("Couldn't upgrade %s"), info->fname);
     message_box(buf);
   }
-  db_handle_db_up(info->hdl);
   rpm_package_list_update_pane(pl);
 }
 static void checksig_func(RpmQuery *info) {
-  check_one_sig(info->fname, 0);
+  check_one_sig(info->fname, 0, NULL);
 }
 
 static void rpm_instpkgs_query(RpmInstallDialog *dlg, GList *files,
@@ -516,10 +500,9 @@ static void rpm_install_pkgs_cb(RpmInstallDialog *dlg, GList *files,
     message_box(_("No packages selected"));
     return;
   }
-  db_handle_db_down(dlg->hdl);
 #ifdef HAVE_RPM_3_0
-  res = do_install(dlg->hdl->root, files, NULL, transFlags, interfaceFlags,
-		   probFilter, gnorpm_update_tree, pl);
+  res = do_install(dlg->hdl->root, files, NULL, transFlags, probFilter,
+		   interfaceFlags, gnorpm_update_tree, pl);
 #else
   res = do_install(dlg->hdl->root, files, NULL, installFlags, interfaceFlags,
 		   gnorpm_update_tree, pl);
@@ -529,25 +512,55 @@ static void rpm_install_pkgs_cb(RpmInstallDialog *dlg, GList *files,
     g_snprintf(buf, 511, _("Install of %d packages failed."), res);
     message_box(buf);
   }
-  db_handle_db_up(dlg->hdl);
   rpm_package_list_update_pane(pl);
 }
+
+static void rpm_upgrade_pkgs_cb(RpmInstallDialog *dlg, GList *files,
+				RpmPackageList *pl) {
+  gint res;
+
+  if (files == NULL) {
+    message_box(_("No packages selected"));
+    return;
+  }
+#ifdef HAVE_RPM_3_0
+  res = do_install(dlg->hdl->root, files, NULL,
+		   transFlags, probFilter, interfaceFlags | INTER_UPGRADE,
+		   gnorpm_update_tree, pl);
+#else
+  res = do_install(dlg->hdl->root, files, NULL,
+		   installFlags | RPMINSTALL_UPGRADE, interfaceFlags,
+		   gnorpm_update_tree, pl);
+#endif
+
+  if (res) {
+    gchar buf[512];
+    g_snprintf(buf, 511, _("Upgrade of %d packages failed."), res);
+    message_box(buf);
+  }
+
+  rpm_package_list_update_pane(pl);
+}
+
 static void rpm_check_sigs_cb(RpmInstallDialog *dlg, GList *files) {
   if (files == NULL) {
     message_box(_("No packages selected"));
     return;
   }
-  check_sigs(files, 0);
+  check_sigs(files, 0, NULL);
 }
 
 static void rpm_install_pkgs(GtkWidget *mi, RpmPackageList *pl) {
   GtkWidget *win;
   win = rpm_install_dialog_new(pl->hdl);
+  rpm_install_dialog_add_default_packages(RPM_INSTALL_DIALOG(win));
   gtk_window_set_title(GTK_WINDOW(win), _("Install"));
   gtk_signal_connect(GTK_OBJECT(win), "query",
 		     GTK_SIGNAL_FUNC(rpm_instpkgs_query), pl);
   gtk_signal_connect(GTK_OBJECT(win), "install",
 		     GTK_SIGNAL_FUNC(rpm_install_pkgs_cb), pl);
+  gtk_signal_connect(GTK_OBJECT(win), "upgrade",
+		     GTK_SIGNAL_FUNC(rpm_upgrade_pkgs_cb), pl);
   gtk_signal_connect(GTK_OBJECT(win), "checksig",
 		     GTK_SIGNAL_FUNC(rpm_check_sigs_cb), NULL);
   gtk_widget_show(win);
@@ -572,7 +585,7 @@ static void mainwin_drop_cb(GnomeApp *app, GdkDragContext *context,
 		       GTK_SIGNAL_FUNC(rpm_check_sigs_cb), NULL);
     list = gnome_uri_list_extract_filenames(selection_data->data);
     for (names = list; names; names = names->next)
-      rpm_install_dialog_add_file(RPM_INSTALL_DIALOG(win), names->data);
+      rpm_install_dialog_add_file(RPM_INSTALL_DIALOG(win), names->data, TRUE);
     gnome_uri_list_free_strings(list);
     gtk_widget_show(win);
     break;
@@ -580,47 +593,6 @@ static void mainwin_drop_cb(GnomeApp *app, GdkDragContext *context,
   }
 }
 
-
-static void rpm_upgrade_pkgs_cb(RpmInstallDialog *dlg, GList *files,
-				RpmPackageList *pl) {
-  gint res;
-
-  if (files == NULL) {
-    message_box(_("No packages selected"));
-    return;
-  }
-  db_handle_db_down(dlg->hdl);
-#ifdef HAVE_RPM_3_0
-  res = do_install(dlg->hdl->root, files, NULL,
-		   transFlags, interfaceFlags | INTER_UPGRADE,
-		   probFilter, gnorpm_update_tree, pl);
-#else
-  res = do_install(dlg->hdl->root, files, NULL,
-		   installFlags | RPMINSTALL_UPGRADE, interfaceFlags,
-		   gnorpm_update_tree, pl);
-#endif
-
-  if (res) {
-    gchar buf[512];
-    g_snprintf(buf, 511, _("Upgrade of %d packages failed."), res);
-    message_box(buf);
-  }
-  db_handle_db_up(dlg->hdl);
-
-  rpm_package_list_update_pane(pl);
-}
-static void rpm_upgrade_pkgs(GtkWidget *mi, RpmPackageList *pl) {
-  GtkWidget *win;
-  win = rpm_install_dialog_new(pl->hdl);
-  gtk_window_set_title(GTK_WINDOW(win), _("Upgrade"));
-  gtk_signal_connect(GTK_OBJECT(win), "query",
-		     GTK_SIGNAL_FUNC(rpm_instpkgs_query), pl->hdl);
-  gtk_signal_connect(GTK_OBJECT(win), "install",
-		     GTK_SIGNAL_FUNC(rpm_upgrade_pkgs_cb), pl);
-  gtk_widget_show(win);
-  /* do_install( ... , installFlags | RPMINSTALL_UPGRADE ) */
-}
-
 #ifdef WITH_RPMFIND
 static void rpm_show_web_find(GtkWidget *mi, RpmPackageList *pl) {
   GtkWidget *win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
@@ -663,7 +635,7 @@ static void rpm_preferences_apply_cb(RpmPropsBox *box, gint page_num,
     uninstallFlags = box->uninstallFlags;
 #endif
     break;
-  case RPM_PROPS_INTERFACE:
+  case RPM_PROPS_PKGLIST:
     if (GTK_TOGGLE_BUTTON(box->asIcons)->active)
       rpm_package_list_set_mode(pl, RPM_PACKAGE_ILIST);
     else
@@ -717,14 +689,12 @@ static void context_uninstall(GtkWidget *mi, gpointer index_p) {
 
   RpmPackageList *pl = gtk_object_get_data(GTK_OBJECT(mi), "package-list");
 
-  db_handle_db_down(pl->hdl);
 #ifdef HAVE_RPM_3_0
   failed = uninstall_one(pl->hdl->root, index, transFlags, probFilter,
 			 interfaceFlags);
 #else
   failed = uninstall_one(pl->hdl->root, index, uninstallFlags, interfaceFlags);
 #endif
-  db_handle_db_up(pl->hdl);
   if (failed) {
     char buf[512];
     g_snprintf(buf, 512, _("%d packages couldn't be uninstalled"), failed);
