@@ -119,10 +119,12 @@ static void rpm_dentry_edit_select_package(GtkWidget *w, RpmDentryEdit *self) {
   gint index = GPOINTER_TO_INT(gtk_object_get_user_data(GTK_OBJECT(w)));
   gint32 i, len, *sizes, max, file_num;
   gint16 *modes;
-  Header h = rpmdbGetRecord(self->hdl->db, index);
+  Header h;
   GnomeDesktopEntry *dentry = gnome_dentry_get_dentry(self->dentry);
   gchar *s, *s2, buf[512], **strlist;
 
+  db_handle_db_up(self->hdl);
+  h = rpmdbGetRecord(self->hdl->db, index);
   /* name on dentry will probably be package name */
   headerGetEntry(h, RPMTAG_NAME, NULL, (void**)&s, NULL);
   if (dentry->name) g_free(dentry->name);
@@ -181,6 +183,7 @@ static void rpm_dentry_edit_select_package(GtkWidget *w, RpmDentryEdit *self) {
   g_free(strlist);
 
   /* XXXX should do something about the icon here -- what ?? */
+  db_handle_db_down(self->hdl);
 
   if (dentry->type) g_free(dentry->type);
   dentry->type = g_strdup("Application");
@@ -197,6 +200,7 @@ static void rpm_dentry_edit_populate_menu(RpmDentryEdit *self) {
   gint index;
   gchar buf[512], *s1, *s2, *s3;
 
+  db_handle_db_up(self->hdl);
   gtk_option_menu_remove_menu(GTK_OPTION_MENU(self->menu));
   menu = gtk_menu_new();
   for (tmp = self->indices; tmp; tmp = tmp->next) {
@@ -223,6 +227,7 @@ static void rpm_dentry_edit_populate_menu(RpmDentryEdit *self) {
     gtk_widget_hide(self->menu);
   else
     gtk_widget_show(self->menu);
+  db_handle_db_down(self->hdl);
 }
 
 static void rpm_dentry_edit_save_cb(GtkWidget *b, RpmDentryEdit *self) {
