@@ -203,6 +203,7 @@ gint rpm_package_list_set_db(RpmPackageList *self, DBHandle *hdl) {
   gchar *group;
 
   self->hdl = hdl;
+  db_handle_db_up(self->hdl);
 
   tree = GTK_PATH_TREE(self->tree);
   gtk_path_tree_freeze(tree);
@@ -216,6 +217,8 @@ gint rpm_package_list_set_db(RpmPackageList *self, DBHandle *hdl) {
     }
   }
   gtk_path_tree_thaw(tree);
+
+  db_handle_db_down(self->hdl);
   return 0;
 }
 
@@ -335,10 +338,13 @@ void rpm_package_list_update_pane(RpmPackageList *self) {
     return;
   }
 
+  db_handle_db_up(self->hdl);
+
   if (rpmdbFindByGroup(self->hdl->db, self->cur_group, &matches)) {
     /* no matches ... */
     gtk_clist_clear(clist);
     gnome_icon_list_clear(ilist);
+    db_handle_db_down(self->hdl);
     return;
   }
 
@@ -377,6 +383,8 @@ void rpm_package_list_update_pane(RpmPackageList *self) {
   dbiFreeIndexRecord(matches);
   gtk_clist_thaw(clist);
   gnome_icon_list_thaw(ilist);
+
+  db_handle_db_down(self->hdl);
 }
 
 static void rpm_package_list_select_row(RpmPackageList *self, gint row,
