@@ -340,6 +340,7 @@ GtkWidget *rpm_query_new_from_index(DBHandle *hdl, guint index) {
 
 GtkWidget *rpm_query_new_from_file(DBHandle *hdl, gchar *file) {
   RpmQuery *self;
+  dbiIndexSet matches;
 
   self = gtk_type_new(rpm_query_get_type());
   if (rpm_query_set_from_file(self, hdl, file)) {
@@ -347,7 +348,14 @@ GtkWidget *rpm_query_new_from_file(DBHandle *hdl, gchar *file) {
     return NULL;
   }
 
-  rpm_query_add_install_buttons(self);
+  rpm_query_add_install_button(self);
+  db_handle_db_up(hdl);
+  if (!rpmdbFindByLabel(hdl->db, self->pkg_name, &matches)) {
+    /* there is a package by this name in the rpm database ... */
+    dbiFreeIndexRecord(matches);
+    rpm_query_add_upgrade_button(self);
+  }
+  db_handle_db_down(hdl);
   rpm_query_add_checksig_button(self);
   rpm_query_add_close_button(self);
   return GTK_WIDGET(self);
@@ -363,13 +371,13 @@ GtkWidget *rpm_query_new_from_rdf(DBHandle *hdl, rpmData *data) {
     return NULL;
   }
 
-  rpm_query_add_install_buttons(self);
+  rpm_query_add_install_button(self);
   rpm_query_add_close_button(self);
   return GTK_WIDGET(self);
 }
 #endif
 
-void rpm_query_add_install_buttons(RpmQuery *self) {
+void rpm_query_add_install_button(RpmQuery *self) {
   GtkWidget *w;
 
   w = gnome_pixmap_button(gnome_stock_pixmap_widget(NULL,
@@ -379,6 +387,10 @@ void rpm_query_add_install_buttons(RpmQuery *self) {
 			    GTK_OBJECT(self));
   gtk_container_add(GTK_CONTAINER(self->action_area), w);
   gtk_widget_show(w);
+}
+
+void rpm_query_add_upgrade_button(RpmQuery *self) {
+  GtkWidget *w;
 
   w = gnome_pixmap_button(gnome_stock_pixmap_widget(NULL,
 				RPM_STOCK_PIXMAP_UPGRADE), _("Upgrade"));
@@ -463,9 +475,13 @@ gint rpm_query_set_from_db(RpmQuery *self, DBHandle *hdl, guint index) {
   Header h;
 
   g_return_val_if_fail(hdl != NULL, 1);
+  db_handle_db_up(hdl);
 
   h = rpmdbGetRecord(hdl->db, index);
-  if (!h) return 1;
+  if (!h) {
+    db_handle_db_down(hdl);
+    return 1;
+  }
   rpm_query_set_from_header(self, h);
   headerFree(h);
 
@@ -479,6 +495,7 @@ gint rpm_query_set_from_db(RpmQuery *self, DBHandle *hdl, guint index) {
   self->source = RPM_QUERY_DB;
   self->hdl = hdl;
   self->index = index;
+  db_handle_db_down(self->hdl);
   return 0;
 }
 
