@@ -89,6 +89,9 @@ void rpm_query_dialog_add_pages(RpmQueryDialog *self, DBHandle *hdl,
 				GList *indices) {
   guint index;
   GtkWidget *page;
+
+  /* this reduces db opens/closes while loading pages */
+  db_handle_db_up(hdl);
   for (; indices != NULL; indices = indices->next) {
     index = GPOINTER_TO_UINT(indices->data);
     page = rpm_query_new_from_index(hdl, index);
@@ -96,6 +99,7 @@ void rpm_query_dialog_add_pages(RpmQueryDialog *self, DBHandle *hdl,
     rpm_query_dialog_add_page(self, RPM_QUERY(page));
   }
   gtk_notebook_set_page(GTK_NOTEBOOK(self->notebook), 0);
+  db_handle_db_down(hdl);
 }
 
 void rpm_query_dialog_add_page_files(RpmQueryDialog *self, DBHandle *hdl,
