@@ -48,8 +48,14 @@ guint rpm_web_find_get_type(void) {
   return webfind_type;
 }
 
+static GtkObjectClass *parent_class;
+
+static void rpm_web_find_destroy(GtkObject *object);
+
 static void rpm_web_find_class_init(RpmWebFindClass *klass) {
-  
+ parent_class = gtk_type_class(gtk_vbox_get_type());
+
+ GTK_OBJECT_CLASS(klass)->destroy = rpm_web_find_destroy;
 }
 
 static gint rpm_web_find_do_search(RpmWebFind *self);
@@ -59,14 +65,10 @@ static void rpm_web_find_tree_select_row(RpmWebFind *self, GtkCTreeNode *node,
 					 gint column);
 
 static void rpm_web_find_install(RpmWebFind *self);
-static void rpm_web_find_upgrade(RpmWebFind *self);
+static void rpm_web_find_transfer(RpmWebFind *self);
 
-static gboolean rpm_web_find_init_tree(RpmWebFind *self) {
-  self->filling_tree = TRUE;
-  gtk_widget_set_sensitive(self->ctree, FALSE);
+static gint init_tree(RpmWebFind *self) {
   rpm_web_find_fill_tree(self, getFullPackageList());
-  gtk_widget_set_sensitive(self->ctree, TRUE);
-  self->filling_tree = FALSE;
   return FALSE;
 }
 
@@ -128,14 +130,23 @@ static void rpm_web_find_init(RpmWebFind *self) {
   gtk_widget_show(self->ctree);
 
   self->info = rpm_query_new();
-  rpm_query_add_install_buttons(RPM_QUERY(self->info));
+  rpm_query_add_install_button(RPM_QUERY(self->info));
+  wid = gnome_pixmap_button(gnome_stock_pixmap_widget(NULL,
+			GNOME_STOCK_PIXMAP_CONVERT), _("Download"));
+  gtk_signal_connect_object(GTK_OBJECT(wid), "clicked",
+			    GTK_SIGNAL_FUNC(rpm_web_find_transfer),
+			    GTK_OBJECT(self));
+  gtk_container_add(GTK_CONTAINER(RPM_QUERY(self->info)->action_area),
+		    wid);
+  gtk_widget_show(wid);
+
   rpm_query_add_close_button(RPM_QUERY(self->info));
   gtk_signal_connect_object(GTK_OBJECT(self->info), "install",
 			    GTK_SIGNAL_FUNC(rpm_web_find_install),
 			    GTK_OBJECT(self));
-  gtk_signal_connect_object(GTK_OBJECT(self->info), "upgrade",
+  /*gtk_signal_connect_object(GTK_OBJECT(self->info), "upgrade",
 			    GTK_SIGNAL_FUNC(rpm_web_find_upgrade),
-			    GTK_OBJECT(self));
+			    GTK_OBJECT(self));*/
   
   gtk_container_set_border_width(GTK_CONTAINER(self->info), GNOME_PAD);
   gtk_paned_add2(GTK_PANED(paned), self->info);
@@ -150,13 +161,12 @@ static void rpm_web_find_init(RpmWebFind *self) {
   self->pkg_p = gdk_pixmap_colormap_create_from_xpm_d(NULL,
 	gtk_widget_get_default_colormap(), &(self->pkg_b), NULL, package_xpm);
 
-  gtk_idle_add((GtkFunction)rpm_web_find_init_tree, self);
-
   self->packageSelected = FALSE;
   self->alt = NULL;
   self->name = NULL;
 
-  self->filling_tree = FALSE;
+  self->fill_tag = 0;
+  self->fill_results = self->fill_pos = NULL;
 
   self->vc_tag = 0;
   self->vc_node = NULL;
@@ -165,6 +175,12 @@ static void rpm_web_find_init(RpmWebFind *self) {
   self->upgrade_cb = NULL;
   self->query_cb = NULL;
   self->checksig_cb = NULL;
+
+  self->oldPkg = NULL;
+  self->newPkg = NULL;
+  self->curPkg = NULL;
+
+  gtk_idle_add((GtkFunction)init_tree, self);
 }
 
 GtkWidget *rpm_web_find_new(RpmPackageList *pl) {
@@ -191,20 +207,26 @@ static gint rpm_web_find_do_search(RpmWebFind *self) {
   GList *results;
   gchar *search;
 
-  if (self->filling_tree) {
-    self->filling_tree = FALSE;
-    return FALSE;
+  /* the version check idle is running -- kill it */
+  if (self->vc_tag)
+    gtk_idle_remove(self->vc_tag);
+  self->vc_tag = 0;
+  self->vc_node = NULL;
+
+  /* a previous fill_tree idle is running -- kill it */
+  if (self->fill_tag) {
+    gtk_widget_set_sensitive(self->ctree, TRUE);
+    gtk_clist_thaw(GTK_CLIST(self->ctree));
+    gtk_idle_remove(self->fill_tag);
+    freeApropos(self->fill_results);
   }
+  self->fill_tag = 0;
+  self->fill_results = self->fill_pos = NULL;
 
-  self->filling_tree = TRUE;
   search = gtk_entry_get_text(GTK_ENTRY(self->entry));
-  gtk_clist_clear(GTK_CLIST(self->ctree));
   results = aproposSearch(search);
-  gtk_widget_set_sensitive(self->ctree, FALSE);
+
   rpm_web_find_fill_tree(self, results);
-  gtk_widget_set_sensitive(self->ctree, TRUE);
-  freeApropos(results);
-  self->filling_tree = FALSE;
   return FALSE;
 }
 
@@ -219,6 +241,7 @@ static gint rpm_web_find_vcheck(RpmWebFind *self) {
   }
   gtk_ctree_get_node_info(GTK_CTREE(self->ctree), self->vc_node, &name,
 			  NULL, NULL, NULL, NULL, NULL, NULL, NULL);
+  db_handle_db_up(self->hdl);
   if (!rpmdbFindByLabel(self->hdl->db, name, &matches)) {
     Header best = NULL;
     gint i;
@@ -246,6 +269,7 @@ static gint rpm_web_find_vcheck(RpmWebFind *self) {
     headerFree(best);
     gtk_ctree_node_set_text(GTK_CTREE(self->ctree), self->vc_node, 1, buf);
   }
+  db_handle_db_down(self->hdl);
   self->vc_node = GTK_CTREE_ROW(self->vc_node)->sibling;
   if (self->vc_node)
     return TRUE;
@@ -253,49 +277,72 @@ static gint rpm_web_find_vcheck(RpmWebFind *self) {
   return FALSE;
 }
 
+static gint rpm_web_find_fill_idle_cb(RpmWebFind *self) {
+  gchar *row[2] = { "", "" };
+  gchar *name = self->fill_pos->data;
+  GtkCTreeNode *child, *bogus;
+
+  row[0] = name;
+  child = gtk_ctree_insert_node(GTK_CTREE(self->ctree), NULL, NULL,
+				row, 2, self->fldclose_p, self->fldclose_b,
+				self->fld_p, self->fld_b, FALSE, FALSE);
+  /* bogus node, so we can catch expand events, and work out nodes only
+   * as needed */
+  bogus = gtk_ctree_insert_node(GTK_CTREE(self->ctree), child, NULL,
+				row, 2, NULL, NULL, NULL, NULL, TRUE, FALSE);
+
+  self->fill_pos = self->fill_pos->next;
+  if (self->fill_pos != NULL)
+    return TRUE;  /* still more rows to process */
+
+  /* we have finished filling the tree */
+  gtk_clist_select_row(GTK_CLIST(self->ctree), 0, 0);
+  gtk_widget_set_sensitive(self->ctree, TRUE);
+  gtk_clist_thaw(GTK_CLIST(self->ctree));
+
+  self->vc_node = gtk_ctree_node_nth(GTK_CTREE(self->ctree), 0);
+  if (self->vc_node)
+    self->vc_tag = gtk_idle_add((GtkFunction)rpm_web_find_vcheck, self);
+
+  freeApropos(self->fill_results);
+  self->fill_tag = 0;
+  self->fill_results = self->fill_pos = NULL;
+
+  return FALSE;
+}
+
 static void rpm_web_find_fill_tree(RpmWebFind *self, GList *results) {
   GList *tmp;
   gchar *row[2] = { "", "" };
 
+  /* the version check idle is running -- kill it */
   if (self->vc_tag)
     gtk_idle_remove(self->vc_tag);
   self->vc_tag = 0;
   self->vc_node = NULL;
 
+  /* a previous fill_tree idle is running -- kill it */
+  if (self->fill_tag) {
+    gtk_widget_set_sensitive(self->ctree, TRUE);
+    gtk_clist_thaw(GTK_CLIST(self->ctree));
+    gtk_idle_remove(self->fill_tag);
+    freeApropos(self->fill_results);
+  }
+  self->fill_tag = 0;
+  self->fill_results = self->fill_pos = NULL;
+
   self->packageSelected = FALSE;
   self->alt = NULL;
   self->name = NULL;
 
-  gtk_clist_freeze(GTK_CLIST(self->ctree));
-  for (tmp = results; tmp; tmp = tmp->next) {
-    char *name = tmp->data;
-    GtkCTreeNode *child, *bogus;
-    
-    row[0] = name;
-    child = gtk_ctree_insert_node(GTK_CTREE(self->ctree), NULL, NULL,
-				    row, 2, self->fldclose_p, self->fldclose_b,
-				    self->fld_p, self->fld_b, FALSE, FALSE);
-    /* bogus node, so we can catch expand events, and work out nodes only
-     * as needed */
-    bogus = gtk_ctree_insert_node(GTK_CTREE(self->ctree), child, NULL,
-				  row, 2, NULL, NULL, NULL, NULL, TRUE, FALSE);
+  self->fill_results = self->fill_pos = results;
+  gtk_clist_clear(GTK_CLIST(self->ctree));
+  if (!self->fill_results)
+    return;
 
-    while (gtk_events_pending())
-      gtk_main_iteration();
-    if (!self->filling_tree) {
-      /* a search was initiated while we were filling the tree */
-      gtk_clist_select_row(GTK_CLIST(self->ctree), 0, 0);
-      gtk_clist_thaw(GTK_CLIST(self->ctree));
-      gtk_idle_add((GtkFunction)rpm_web_find_do_search, self);
-      g_message("results display was interrupted");
-      return;
-    }
-  }
-  gtk_clist_select_row(GTK_CLIST(self->ctree), 0, 0);
-  gtk_clist_thaw(GTK_CLIST(self->ctree));
-  self->vc_node = gtk_ctree_node_nth(GTK_CTREE(self->ctree), 0);
-  if (self->vc_node)
-    self->vc_tag = gtk_idle_add((GtkFunction)rpm_web_find_vcheck, self);
+  gtk_widget_set_sensitive(self->ctree, FALSE);
+  gtk_clist_freeze(GTK_CLIST(self->ctree));
+  self->fill_tag = gtk_idle_add((GtkFunction)rpm_web_find_fill_idle_cb, self);
 }
 
 static void rpm_web_find_tree_expand(RpmWebFind *self, GtkCTreeNode *node) {
@@ -319,6 +366,9 @@ static void rpm_web_find_tree_expand(RpmWebFind *self, GtkCTreeNode *node) {
     gtk_widget_set_sensitive(self->ctree, FALSE);
     alts = alternateSearch(name);
     alts = sortAlternates(alts, NULL, NULL);
+    db_handle_db_up(self->hdl);
+    statAlternates(self->hdl->db, alts);
+    db_handle_db_down(self->hdl);
     if (crow->children)
       gtk_ctree_remove_node(GTK_CTREE(self->ctree), crow->children);
 
@@ -335,6 +385,43 @@ static void rpm_web_find_tree_expand(RpmWebFind *self, GtkCTreeNode *node) {
       gtk_ctree_node_set_row_data_full(GTK_CTREE(self->ctree), child,
 				       alt, (GtkDestroyNotify)
 				       freeRpmPackageAlternate);
+      if (alt->status == OLD_PACKAGE) {
+	if (self->oldPkg == NULL) {
+	  GdkColor colour;
+	  gchar *colour_name = gnome_config_get_string("/gnorpm/Install/oldPkg=grey50");
+	  self->oldPkg = gtk_style_copy(gtk_widget_get_style(self->ctree));
+	  if (gdk_color_parse(colour_name, &colour) &&
+	      gdk_color_alloc(gtk_widget_get_colormap(self->ctree), &colour))
+	    self->oldPkg->fg[GTK_STATE_NORMAL] = colour;
+	  g_free(colour_name);
+	}
+	gtk_ctree_node_set_row_style(GTK_CTREE(self->ctree), child,
+				     self->oldPkg);
+      } else if (alt->status == NEW_PACKAGE) {
+	if (self->newPkg == NULL) {
+	  GdkColor colour;
+	  gchar *colour_name = gnome_config_get_string("/gnorpm/Install/newPkg=blue");
+	  self->newPkg = gtk_style_copy(gtk_widget_get_style(self->ctree));
+	  if (gdk_color_parse(colour_name, &colour) &&
+	      gdk_color_alloc(gtk_widget_get_colormap(self->ctree), &colour))
+	    self->newPkg->fg[GTK_STATE_NORMAL] = colour;
+	  g_free(colour_name);
+	}
+	gtk_ctree_node_set_row_style(GTK_CTREE(self->ctree), child,
+				     self->newPkg);
+      } else if (alt->status == CURRENT_PACKAGE) {
+	if (self->curPkg == NULL) {
+	  GdkColor colour;
+	  gchar *colour_name = gnome_config_get_string("/gnorpm/Install/curPkg=green4");
+	  self->curPkg = gtk_style_copy(gtk_widget_get_style(self->ctree));
+	  if (gdk_color_parse(colour_name, &colour) &&
+	      gdk_color_alloc(gtk_widget_get_colormap(self->ctree), &colour))
+	    self->curPkg->fg[GTK_STATE_NORMAL] = colour;
+	  g_free(colour_name);
+	}
+	gtk_ctree_node_set_row_style(GTK_CTREE(self->ctree), child,
+				     self->curPkg);
+      }
     }
     g_list_free(alts);
     gtk_widget_set_sensitive(self->ctree, TRUE);
@@ -361,16 +448,18 @@ static void rpm_web_find_tree_select_row(RpmWebFind *self, GtkCTreeNode *node,
   gtk_widget_queue_resize(self->info);
 }
 
-static void rpm_web_find_get_packages(RpmWebFind *self, gboolean upgrade) {
+static gboolean rpm_web_find_get_packages(RpmWebFind *self, GList **local) {
   gchar *name;
   GList *urls = NULL, *tmp, *items = NULL, *localfiles = NULL;
   GtkWidget *win, *box, *list, *w;
   gint res;
 
+  db_handle_db_up(self->hdl);
   if (self->packageSelected)
     urls = resolveAlternate(self->alt, self->hdl->db);
   else
     urls = resolveName(self->name, self->hdl->db);
+  db_handle_db_down(self->hdl);
 
   if (!urls) {
     win = gnome_message_box_new(_("No packages to download"),
@@ -380,7 +469,7 @@ static void rpm_web_find_get_packages(RpmWebFind *self, gboolean upgrade) {
 		GTK_WINDOW(gtk_widget_get_toplevel(GTK_WIDGET(self))));
     set_icon(win);
     gtk_widget_show(win);
-    return;
+    return FALSE;
   }
 
   win = gnome_dialog_new(_("Download files?"), GNOME_STOCK_BUTTON_YES,
@@ -420,7 +509,7 @@ static void rpm_web_find_get_packages(RpmWebFind *self, gboolean upgrade) {
   if (res == 1) {
     g_list_foreach(urls, (GFunc)g_free, NULL);
     g_list_free(urls);
-    return;
+    return FALSE;
   }
 
   for (tmp = urls; tmp; tmp = tmp->next) {
@@ -432,25 +521,37 @@ static void rpm_web_find_get_packages(RpmWebFind *self, gboolean upgrade) {
       g_list_free(urls);
       g_list_foreach(localfiles, (GFunc)g_free, NULL);
       g_list_free(localfiles);
-      return;
+      return FALSE;
     }
     localfiles = g_list_append(localfiles, file);
   }
   g_list_foreach(urls, (GFunc)g_free, NULL);
   g_list_free(urls);
 
-  win = rpm_install_dialog_new(self->hdl);
-  if (upgrade) {
-    gtk_window_set_title(GTK_WINDOW(win), _("Upgrade"));
-    if (self->upgrade_cb)
-      gtk_signal_connect(GTK_OBJECT(win), "install",
-			 self->upgrade_cb, self->pl);
-  } else {
-    gtk_window_set_title(GTK_WINDOW(win), _("Install"));
-    if (self->install_cb)
-      gtk_signal_connect(GTK_OBJECT(win), "install",
-			 self->install_cb, self->pl);
+  if (local)
+    *local = localfiles;
+  else {
+    g_list_foreach(localfiles, (GFunc)g_free, NULL);
+    g_list_free(localfiles);
   }
+  return TRUE;
+}
+
+static void rpm_web_find_install(RpmWebFind *self) {
+  GtkWidget *win;
+  GList *localfiles, *tmp;
+
+  if (!rpm_web_find_get_packages(self, &localfiles))
+    return;
+
+  win = rpm_install_dialog_new(self->hdl);
+  gtk_window_set_title(GTK_WINDOW(win), _("Install"));
+  if (self->install_cb)
+    gtk_signal_connect(GTK_OBJECT(win), "install",
+		       self->install_cb, self->pl);
+  if (self->upgrade_cb)
+    gtk_signal_connect(GTK_OBJECT(win), "upgrade",
+		       self->upgrade_cb, self->pl);
   if (self->query_cb)
     gtk_signal_connect(GTK_OBJECT(win), "query",
 		       self->query_cb, self->pl);
@@ -459,17 +560,38 @@ static void rpm_web_find_get_packages(RpmWebFind *self, gboolean upgrade) {
 		       self->checksig_cb, self->hdl);
 
   for (tmp = localfiles; tmp; tmp = tmp->next)
-    rpm_install_dialog_add_file(RPM_INSTALL_DIALOG(win), tmp->data);
+    rpm_install_dialog_add_file(RPM_INSTALL_DIALOG(win), tmp->data, TRUE);
   g_list_foreach(localfiles, (GFunc)g_free, NULL);
   g_list_free(localfiles);
   gtk_widget_show(win);
 }
 
-static void rpm_web_find_install(RpmWebFind *self) {
-  rpm_web_find_get_packages(self, FALSE);
-}
-static void rpm_web_find_upgrade(RpmWebFind *self) {
-  rpm_web_find_get_packages(self, TRUE);
+static void rpm_web_find_transfer(RpmWebFind *self) {
+  rpm_web_find_get_packages(self, NULL);
 }
 
+static void rpm_web_find_destroy(GtkObject *object) {
+  RpmWebFind *self = RPM_WEB_FIND(object);
 
+  if (self->oldPkg) gtk_style_unref(self->oldPkg);
+  if (self->newPkg) gtk_style_unref(self->newPkg);
+  if (self->curPkg) gtk_style_unref(self->curPkg);
+
+  gdk_pixmap_unref(self->fld_p);      gdk_bitmap_unref(self->fld_b);
+  gdk_pixmap_unref(self->fldclose_p); gdk_bitmap_unref(self->fldclose_b);
+  gdk_pixmap_unref(self->pkg_p);      gdk_bitmap_unref(self->pkg_b);
+  
+  /* the version check idle is running -- kill it */
+  if (self->vc_tag)
+    gtk_idle_remove(self->vc_tag);
+
+  /* a previous fill_tree idle is running -- kill it */
+  if (self->fill_tag) {
+    gtk_clist_thaw(GTK_CLIST(self->ctree));
+    gtk_idle_remove(self->fill_tag);
+    freeApropos(self->fill_results);
+  }
+
+  if (parent_class->destroy)
+    (* parent_class->destroy)(object);
+}
