@@ -23,12 +23,12 @@
 #include <string.h>
 #include "misc.h"
 
-void verify_packages(DBHandle *hdl, GList *indices, gint omitMask) {
+GtkWidget *verify_packages(DBHandle *hdl, GList *indices, gint omitMask) {
   Header h;
   GtkWidget *win, *vbox, *pkg_name, *fname, *swin, *clist, *button;
   gchar buf1[512], buf2[512], *s1, *s2, *s3, **files;
   gchar *headings[] = {N_("Package"), N_("File"), N_("Problem")}, *row[3];
-  guint index, i, count;
+  guint index, i, count, failCount = 0;
   gint result;
 
   headings[0] = _(headings[0]);
@@ -42,7 +42,7 @@ void verify_packages(DBHandle *hdl, GList *indices, gint omitMask) {
   gtk_box_pack_start(GTK_BOX(GTK_DIALOG(win)->vbox), vbox, TRUE, TRUE, 0);
   gtk_widget_show(vbox);
 
-  pkg_name = gtk_label_new("<pkg_name>");
+  pkg_name = gtk_label_new(_("No packages selected"));
   gtk_box_pack_start(GTK_BOX(vbox), pkg_name, FALSE, TRUE, 0);
   gtk_widget_show(pkg_name);
   fname = gtk_label_new("<file>");
@@ -77,6 +77,7 @@ void verify_packages(DBHandle *hdl, GList *indices, gint omitMask) {
 
   gtk_widget_show(win);
 
+  db_handle_db_up(hdl);
   for (; indices != NULL; indices = indices->next) {
     index = GPOINTER_TO_UINT(indices->data);
 
@@ -113,6 +114,7 @@ void verify_packages(DBHandle *hdl, GList *indices, gint omitMask) {
 						    _(", device file type"));
 	  }
 	  gtk_clist_append(GTK_CLIST(clist), row);
+	  failCount++;
 	}
       }
       free(files);
@@ -126,17 +128,26 @@ void verify_packages(DBHandle *hdl, GList *indices, gint omitMask) {
 	row[1] = _("*script*");
 	row[2] = _("script problem");
 	gtk_clist_append(GTK_CLIST(clist), row);
+	failCount++;
       }
     }
     headerFree(h);
   }
-  gtk_label_set(GTK_LABEL(fname), "");
+  db_handle_db_down(hdl);
+  if (failCount) {
+    g_snprintf(buf1, 511, _("%d problems found."), failCount);
+    gtk_label_set(GTK_LABEL(fname), buf1);
+  } else
+    gtk_label_set(GTK_LABEL(fname), _("No problems found."));
   gtk_widget_set_sensitive(button, TRUE);
+  return win;
 }
 
-void verify_one(DBHandle *hdl, guint index, gint omitMask) {
+GtkWidget *verify_one(DBHandle *hdl, guint index, gint omitMask) {
   GList *indices;
+  GtkWidget *ret;
   indices = g_list_append(NULL, GUINT_TO_POINTER(index));
-  verify_packages(hdl, indices, omitMask);
+  ret = verify_packages(hdl, indices, omitMask);
   g_list_free(indices);
+  return ret;
 }
